/*
 * This program calculates days between dates
 * @author hornick
 */
package se1021.dateCalculator.YOURNAME;

import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;

@SuppressWarnings("serial") // ignore warnings about file serialization
public class DateCalculatorApp extends JFrame {
	private JLabel lblNumberOfDaysResult; 	// label for number of days between dates
	private JTextField txtStartDate; 		// text input field for Start Date
	private JTextField txtEndDate; 			// text input field for End Date
	

	/**
	 * main method; creates an instance of itself; thereby creating the UI
	 * @param args standard command line parameters
	 */
	public static void main(String[] args) {
		DateCalculatorApp me = new DateCalculatorApp();
		me.setVisible(true);
	}

	/*
	 * display an error message to the user of this application
	 * @param msg - the error message to display
	 */
	private void displayErrorMessage(String msg) {
		JOptionPane.showMessageDialog(this, msg);
	}

	/*
	 * Updates the label field in the UI that displays the result
	 * of computing the days between dates
	 * @param numDays - the number of days that gets displayed
	 */
	private void updateResult( int numDays ) {
		lblNumberOfDaysResult.setText(String.format("%d", numDays) );
	}

	/**
	 * Default constructor; initializes the UI and creates an event handler
	 *
	 */
	public DateCalculatorApp() {
		
		// create the nested event handler that will respond to incoming events
		EventHandler evtHandler = new EventHandler();

		// create the containing window and set its size etc
		setTitle("SE1021 Date Calculator"); 
		setSize(300, 200);
		setLocation(400, 400);
		setResizable(false);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// Retrieves the window's contentPane and sets the layout to absolute positioning
		Container contentPane = getContentPane();
		contentPane.setLayout(null);

		//Creation of "Calculate" button
		JButton btnCalc;
		btnCalc = new JButton ("Calculate"); 
		btnCalc.setActionCommand("calc");
		btnCalc.setBounds(10, 130, 170, 30);
		btnCalc.addActionListener(evtHandler);	// subscribe to events
		contentPane.add(btnCalc);

		//Creation of "Quit" button
		JButton btnQuit;
		btnQuit = new JButton ("Quit");
		btnQuit.setBounds(190, 130, 95, 30);
		btnQuit.addActionListener(evtHandler);	// subscribe to events
		btnQuit.setActionCommand("close");
		contentPane.add(btnQuit);

		//Creation of "Start Date" label
		JLabel lblDateOne = new JLabel ("Start Date:");
		lblDateOne.setBounds(10, 10, 170, 30);
		contentPane.add(lblDateOne);

		//Creation of "Start Date" input field
		txtStartDate = new JTextField();
		txtStartDate.setBounds(150, 10, 135, 30);
		txtStartDate.setActionCommand("start");
		txtStartDate.addActionListener(evtHandler);
		contentPane.add(txtStartDate);

		//Creation of "End Date" label
		JLabel lblDateTwo = new JLabel ("End Date:");
		lblDateTwo.setBounds(10, 50, 170, 30);
		contentPane.add(lblDateTwo);

		//Creation of "End Date" input field
		txtEndDate = new JTextField();
		txtEndDate.setBounds(150, 50, 135, 30);
		txtEndDate.setActionCommand("end");
		txtEndDate.addActionListener(evtHandler);
		contentPane.add(txtEndDate);

		// Creation of "Days Between Dates" label
		JLabel lblDaysBetween = new JLabel ("Days Between Dates:");
		lblDaysBetween.setBounds(10, 90, 170, 30);
		contentPane.add(lblDaysBetween);

		// Creation of label used to output answer
		lblNumberOfDaysResult = new JLabel ("0");
		lblNumberOfDaysResult.setBounds(150, 90, 170, 30);
		contentPane.add(lblNumberOfDaysResult);

	} // end of Constructor method

	/*
	 * This is a nested inner class that handles events
	 * for the DateCalculatorApp  
	 * @author hornick
	 *
	 */
	private class EventHandler implements ActionListener {

		// Constructor; does nothing. 
		private EventHandler() {
		}

		// ActionListener event handler for this class
		// @param event - the ActionEvent object that caused the event
		public void actionPerformed(ActionEvent event) {

			String cmd = event.getActionCommand(); 	// get the Action Command that caused this event

			if (cmd.equals("close")) 	// the "Quit" button was pressed
			{
				dispose();  	// Destroys the JFrame and everything in it
				System.exit(0); 		//Closes the entire program
			}

			// checks whether the "Calculate" button or ENTER key was pressed while inside one of the text fields
			if ( cmd.equals("start") || cmd.equals("calc") || cmd.equals("end") )
			{
				handleCalculate();	// delegate the actual handling of the event to another method
			}

		}
		
		/**
		 * handle the actual processing of the Start and End Date entries;
		 * report errors to the user when they occur.
		 */
		private void handleCalculate() {
			String strStartDate = txtStartDate.getText(); //String used to retrieve information from fields
			String strEndDate = txtEndDate.getText(); //String used to retrieve information from fields
			
			int startMonth, startDay, startYear, endMonth, endDay, endYear;

// TODO Surround the following 8 statements with try-catch to trap Exceptions propagated from DateStringConverter. 
// TODO	Using displayErrorMessage, report detailed errors on exceptions indicated exactly which field (Start or End date)
			DateStringConverter startDate = new DateStringConverter(strStartDate); // process start date
			startMonth = startDate.getMonth(); 
			startDay = startDate.getDay();
			startYear = startDate.getYear();

			DateStringConverter endDate = new DateStringConverter(strEndDate); // process end date
			endMonth = endDate.getMonth(); 
			endDay = endDate.getDay();
			endYear = endDate.getYear();

// TODO catch Exceptions thrown from within DayToDateConverter; report errors on exceptions,
// TODO Using displayErrorMessage, report exactly the cause of the error (invalid values or End preceded Start Date),
// TODO or update the JLabel that displays the results if no error occurs
			DaysCalculator ddc = new DaysCalculator(startDay, startMonth, startYear, endDay, endMonth, endYear);
			int numDays = ddc.getDaysBetweenDates();
			updateResult( numDays );
		}

	}	// end of EventHandler class

} // end of the DateCalculatorApp class
